#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pymipago` package."""


from xml.etree.ElementTree import ParseError
from pymipago.utils import _build_payment_code_notebook_60
from pymipago.utils import _calculate_payment_identification_notebook_60
from pymipago.utils import _calculate_reference_number_with_control_digits_notebook_60 # noqa
from pymipago.utils import _parse_initialization_response
from pymipago.exceptions import InvalidReferenceNumber

import datetime
import unittest


class TestUtils(unittest.TestCase):
    """Tests for `utils` package."""

    def test_calculate_payment_identification_notebook_60(self):
        date = datetime.date(2018, 1, 1)
        suffix = '521'
        value = '1521188001'
        calculated_value = _calculate_payment_identification_notebook_60(date, suffix) # noqa
        self.assertEqual(value, calculated_value)

    def test_calculate_reference_number_with_control_digits_notebook_60(self):
        sender = '123456'
        reference_number = '1234567890'
        payment_identification = '123456879012'
        quantity = '1200'

        sum_value = 0
        sum_value += int(sender) * 76
        sum_value += int(reference_number) * 9
        sum_value += (int(payment_identification) + int(quantity) - 1) * 55

        division = sum_value / 97.0
        _, decimals = str(division).split('.')
        if len(decimals) > 1:
            first_two_decimals = decimals[:2]
        else:
            first_two_decimals = decimals[:1] + '0'

        control_digits = 100 - int(first_two_decimals)
        value = '{}{}'.format(reference_number, control_digits)

        calculated_value = _calculate_reference_number_with_control_digits_notebook_60(  # noqa
            sender, reference_number, payment_identification, quantity
        )
        self.assertEqual(value, calculated_value)

    def test_calculate_reference_number_with_invalid_reference_number(self):
        sender = '123456'
        reference_number = '1234567890132132'
        payment_identification = '123456879012'
        quantity = '1200'

        with self.assertRaises(InvalidReferenceNumber):
            _calculate_reference_number_with_control_digits_notebook_60(
                sender, reference_number, payment_identification, quantity
            )

    def test_build_payment_code_notebook_60(self):
        sender = '123456'
        reference_number = '123456789012'
        payment_identification = '1234568790'
        quantity = '1000'

        value = '90521{sender:0>6}{reference_number:0>12}{payment_identification:0>10}{quantity:0>8}0'.format( # noqa
            sender=sender,
            reference_number=reference_number,
            payment_identification=payment_identification,
            quantity=quantity,
        )

        calculated_value = _build_payment_code_notebook_60(
            sender, reference_number, payment_identification, quantity)
        self.assertEqual(value, calculated_value)

    def test_parse_initialization_response_invalid_xml(self):
        xmlvalue = ''
        with self.assertRaises(ParseError):
            _, _ = _parse_initialization_response(xmlvalue)

    def test_parse_initialization_response_when_payment_is_incorrect(self):
        payment_id = None
        error = '''<?xml version='1.0' encoding='ISO-8859-1'?><rpcCallResult><dataType code='16'>java.lang.reflect.InvocationTargetException</dataType><returnValue>java.lang.reflect.InvocationTargetException</returnValue></rpcCallResult>'''  # noqa

        xmlvalue = '''<?xml version='1.0' encoding='ISO-8859-1'?><rpcCallResult><dataType code='16'>java.lang.reflect.InvocationTargetException</dataType><returnValue>java.lang.reflect.InvocationTargetException</returnValue></rpcCallResult>'''.format(payment_id=error)  # noqa

        calculated_payment_id, calculated_error = _parse_initialization_response(xmlvalue)  # noqa

        self.assertEqual(payment_id, calculated_payment_id)
        self.assertEqual(error, calculated_error)

    def test_parse_initialization_response_when_payment_validation_fails(self):
        payment_id = None
        error = 'La longitud de la referencia no es correcta'

        xmlvalue = '''<?xml version='1.0' encoding='ISO-8859-1'?><rpcCallResult><dataType code='3'>java.lang.String</dataType><returnValue><paymentRequestData><peticionesPago><peticionPago id='0'><domiciliacion><permitir>false</permitir></domiciliacion><validacion><codigoError>referencia_longitud</codigoError><mensajeValidacion><![CDATA[La longitud de la referencia no es correcta]]></mensajeValidacion><valido>false</valido></validacion><backend><enabled>false</enabled></backend><datosPago><codigo>905214811661234567890111111002188121000011600</codigo><cpr>9052180</cpr><tipo>002</tipo><periodosPago><periodoPago id='periodoNormal'><importe>1160</importe><validarFechaFin>true</validarFechaFin><identificacion>1002188121</identificacion><fechaFin>010518</fechaFin><activo>false</activo></periodoPago></periodosPago><formato>521</formato><emisor>481166</emisor><validar>1</validar><referencia>12345678901111186</referencia></datosPago></peticionPago></peticionesPago></paymentRequestData></returnValue></rpcCallResult>
'''.format(payment_id=error) # noqa

        calculated_payment_id, calculated_error = _parse_initialization_response(xmlvalue)  # noqa

        self.assertEqual(payment_id, calculated_payment_id)
        self.assertEqual(error, calculated_error)

    def test_parse_initialization_response_when_payment_is_correct(self):
        payment_id = ''
        error = None
        xmlvalue = '''<?xml version='1.0' encoding='ISO-8859-1'?><rpcCallResult><dataType code='3'>java.lang.String</dataType><returnValue><paymentRequestData><peticionesPago><peticionPago id='{payment_id}'><imagenes><imagen id='logoEmisor'><url><![CDATA[]]></url></imagen></imagenes><descripcion><eu>Reserva de instalaciones deportivas</eu><es>Reserva de instalaciones deportivas</es></descripcion><domiciliacion><permitir>false</permitir></domiciliacion><aplicacion codigo='a'><nombre><eu>a</eu><es>a</es></nombre></aplicacion><backend><enabled>false</enabled></backend><datosPago><codigo>{payment_id}</codigo><cpr>9052180</cpr><tipo>002</tipo><periodosPago><periodoPago id='periodoNormal'><importe>1160</importe><validarFechaFin>true</validarFechaFin><identificacion>1002188121</identificacion><fechaFin>010518</fechaFin><activo>false</activo></periodoPago></periodosPago><formato>521</formato><emisor>481166</emisor><validar>1</validar><referencia>123456789017</referencia></datosPago><emisor><codigoPostal></codigoPostal><territorio></territorio><nombre><eu>Amorebieta-Etxano - Ametx organismo aut�nomo eu</eu><es>Amorebieta-Etxano - Ametx organismo aut�nomo</es></nombre><codigo>04800687</codigo><municipio></municipio><cif>Q4800687H</cif><calle></calle></emisor></peticionPago></peticionesPago></paymentRequestData></returnValue></rpcCallResult>
        '''.format(payment_id=payment_id) # noqa
        calculated_payment_id, calculated_error = _parse_initialization_response(xmlvalue)  # noqa

        self.assertEqual(payment_id, calculated_payment_id)
        self.assertEqual(error, calculated_error)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pymipago` package."""

import datetime
import os
import unittest

from pymipago import make_payment_request
from pymipago.exceptions import InvalidCPRValue
from pymipago.exceptions import InvalidFormatValue
from pymipago.exceptions import InvalidReferenceNumber


class TestPymipago(unittest.TestCase):
    """Tests for `pymipago` package."""

    def test_invalid_cpr_raises_exception(self):
        cpr = '010101'
        sender = '123456'
        format = '521'
        suffix = '002'
        reference_number = '1234567890'
        payment_limit_date = datetime.datetime.now()
        quantity = '1100'
        language = 'eu'
        return_url = 'http://localhost:8000'
        payment_modes = ['01']

        with self.assertRaises(InvalidCPRValue):
            make_payment_request(
                cpr,
                sender,
                format,
                suffix,
                reference_number,
                payment_limit_date,
                quantity,
                language,
                return_url,
                payment_modes,
                test_environment=True)

    def test_invalid_format_raises_exception(self):
        cpr = '9052180'
        sender = '123456'
        format = '855'
        suffix = '002'
        reference_number = '1234567890'
        payment_limit_date = datetime.datetime.now()
        quantity = '1100'
        language = 'eu'
        return_url = 'http://localhost:8000'
        payment_modes = ['01']

        with self.assertRaises(InvalidFormatValue):
            make_payment_request(
                cpr,
                sender,
                format,
                suffix,
                reference_number,
                payment_limit_date,
                quantity,
                language,
                return_url,
                payment_modes,
                test_environment=True)

    def test_invalid_formated_reference_number_raises_exception(self):
        cpr = '9052180'
        sender = '123456'
        format = '521'
        suffix = '002'
        reference_number = '12345678'
        payment_limit_date = datetime.datetime.now()
        quantity = '1100'
        language = 'eu'
        return_url = 'http://localhost:8000'
        payment_modes = ['01']

        with self.assertRaises(InvalidReferenceNumber):
            make_payment_request(
                cpr,
                sender,
                format,
                suffix,
                reference_number,
                payment_limit_date,
                quantity,
                language,
                return_url,
                payment_modes,
                test_environment=True)

    def test_correct_payment_request_with_minimal_options(self):
        cpr = '9052180'
        sender = '481166'
        format = '521'
        suffix = '002'
        reference_number = '8123456789'
        payment_limit_date = datetime.datetime.now()
        quantity = '1100'
        language = 'eu'
        return_url = 'http://localhost:8000'
        payment_modes = ['01']

        html, payment_code = make_payment_request(
            cpr,
            sender,
            format,
            suffix,
            reference_number,
            payment_limit_date,
            quantity,
            language,
            return_url,
            payment_modes,
            test_environment=True)

        self.assertTrue(html.find(payment_code) != -1)
        self.assertTrue(payment_code.isdigit())

    def test_correct_payment_request_with_full_options(self):
        cpr = '9052180'
        sender = '481166'
        format = '521'
        suffix = '002'
        reference_number = '8123456789'
        payment_limit_date = datetime.datetime.now()
        quantity = '1100'
        language = 'eu'
        return_url = 'http://localhost:8000'
        payment_modes = ['01']

        extra = {
            'message_1': {
                'eu': 'Message: message_1 in Basque (eu)',
                'es': 'Message: message_1 in Spanish (es)'
            },
            'message_2': {
                'eu': 'Message: message_2 in Basque (eu)',
                'es': 'Message: message_2 in Spanish (es)'
            },
            'message_3': {
                'eu': 'Message: message_3 in Basque (eu)',
                'es': 'Message: message_3 in Spanish (es)'
            },
            'message_4': {
                'eu': 'Message: message_4 in Basque (eu)',
                'es': 'Message: message_4 in Spanish (es)'
            },
            'message_payment_title': {
                'eu': 'Message: message_payment_title in Basque (eu)',
                'es': 'Message: message_payment_title in Spanish (es)'
            },
            'mipago_payment_description': {
                'eu': 'Message: mipago_payment_description in Basque (eu)',
                'es': 'Message: mipago_payment_description in Spanish (es)'
            },
            'logo_1_url': '/data/this-is-a-demo-url-of-logo_1_url.gif',
            'logo_2_url': '/data/this-is-a-demo-url-of-logo_2_url.gif',
            'pdf_xslt_url': '/data/this-is-a-demo-url-of-pdf_xslt_url.xsl',
        }

        html, payment_code = make_payment_request(
            cpr,
            sender,
            format,
            suffix,
            reference_number,
            payment_limit_date,
            quantity,
            language,
            return_url,
            payment_modes,
            extra=extra,
            test_environment=True)

        self.assertTrue(html.find(payment_code) != -1)
        self.assertTrue(payment_code.isdigit())



# -*- coding: utf-8 -*-
"""util functions used by the main module"""

import xml.etree.ElementTree as ET
from .exceptions import InvalidReferenceNumber


def _calculate_payment_identification_notebook_60(limit_date, suffix):
    """Payment identification is calculated concatenating 5 values:

        - A constant: '1'
        - The suffix passed as parameter as a 3 digit string: '521'
        - The last 2 digits of the year of the date passed
            as parameter: '18'
        - The last digit of the year of the date passed
            as parameter: '8'
        - The ordinal day of the date passed as paramenter as a
            3 digit string: '521'
    """
    period = '1'
    year_two_digits = str(limit_date.year)[2:]
    year_last_digit = year_two_digits[-1]
    year_ordinal_day = limit_date.timetuple().tm_yday
    return '{period}{suffix:0>3}{year_two_digits}{year_last_digit}{year_ordinal_day:0>3}'.format( # noqa
        period=period,
        suffix=suffix,
        year_two_digits=year_two_digits,
        year_last_digit=year_last_digit,
        year_ordinal_day=year_ordinal_day,
    )

def _calculate_reference_number_with_control_digits_notebook_60(sender, reference_number, payment_identification, quantity): # noqa
    """ Control digits for the reference number are calculated as follows:
        - a: Multiply the sender value converted to an integer value by 76
        - b: Multiply the reference value converted to an integer value by 9
        - c: Sum the payment_identification converted to an integer
            value and the quantity value converted to an integer value
            and deduct 1.
        - d: Multiply the c value by 55.
        - e: sum a, b and d
        - Divide e by 97 and take the decimal values.
        - f: take the first 2 decimal values (add a 0 as a second digit if
            the division result creates just one decimal)
        - g: deduct f from 100.
        - Concatenate the reference number and g and create a 12 digit value
    """
    if len(reference_number) != 10:
        raise InvalidReferenceNumber

    total = int(sender)*76
    total += int(reference_number)*9
    total += (int(payment_identification)-1+int(quantity))*55

    division_result = total / 97.0
    _, decimals = str(division_result).split('.')

    first_two_decimals = '{:0<2}'.format(decimals)[:2]
    control_digits = 100 - int(first_two_decimals)

    return '{reference_number}{control_digits}'.format(
        reference_number=reference_number, control_digits=control_digits)

def _build_payment_code_notebook_60(sender, reference_number, payment_identification, quantity): # noqa
    """ Payment code is calculated concatenating 6 values:

        - A constant that represents this payment mode: '90521'
        - A 6 digit sender code: '123456'
        - A 12 digit reference number: '123456789012'
        - A 10 digit payment identification number: '1234567890'
        - A 8 digit value representing the number of euro cents to
            be payed: '0000001000'
        - A constant value: '0'
    """
    return '90521{sender:0>6}{reference_number:0>12}{payment_identification:0>10}{quantity:0>8}0'.format( # noqa
        sender=sender,
        reference_number=reference_number,
        payment_identification=payment_identification,
        quantity=quantity,
    )


def _parse_initialization_response(xmlresponse):
    root = ET.fromstring(xmlresponse)
    petition = root.find('.//peticionPago')
    if petition is not None:
        pago_id = petition.get('id')
        if petition.find('.//validacion'):
            # ERROR
            return None, petition.find('.//mensajeValidacion').text

        return pago_id, None

    return None, xmlresponse



# -*- coding: utf-8 -*-
from .constants import INITIALIZATION_XML
from .constants import LOGO_1_TEMPLATE
from .constants import LOGO_2_TEMPLATE
from .constants import LOGO_WRAPPER_TEMPLATE
from .constants import MESSAGE_1_TEMPLATE
from .constants import MESSAGE_2_TEMPLATE
from .constants import MESSAGE_3_TEMPLATE
from .constants import MESSAGE_4_TEMPLATE
from .constants import MESSAGE_PAYMENT_DESCRIPTION
from .constants import MESSAGE_PAYMENT_TITLE
from .constants import PDF_XSLT_TEMPLATE
from .constants import PRESENTATION_XML
from .constants import PROTOCOL_DATA_XML
from .exceptions import InvalidCPRValue
from .exceptions import InvalidFormatValue
from .exceptions import InvalidRegistration
from .utils import _build_payment_code_notebook_60
from .utils import _calculate_payment_identification_notebook_60
from .utils import _calculate_reference_number_with_control_digits_notebook_60
from .utils import _parse_initialization_response

import requests


def make_payment_request(
        cpr, sender, format, suffix, reference_number, payment_limit_date, quantity,
        language, return_url, payment_modes=['01', '02'], test_environment=False, extra={}):

    """This method creates an XML file and creates a payment request on the
       Government platform in order to have the basis to be shown to the end
       user.

       According to the payment platform specs, after the registration, an HTML
       file is created which must be shown to the user. This HTML file has an
       "auto-refresh" feature which allows to redirect the user to the payment
       platform, where all the data of the payment is already entered.

       There, the enduser only has to select the bank of his choice to complete
       the payment.

       After completing the payment the user will be redirected to the
       `return_url`.

       See the documentation for more information about the parameters

    """

    if test_environment:
        from .constants import TEST_ENVIRON_INITIALIZATION_URL as INITIALIZATION_URL # noqa
        from .constants import TEST_ENVIRON_SERVICE_URL as SERVICE_URL
    else:
        from .constants import PROD_ENVIRON_INITIALIZATION_URL as INITIALIZATION_URL # noqa
        from .constants import PROD_ENVIRON_SERVICE_URL as SERVICE_URL

    if cpr != '9052180':
        raise InvalidCPRValue('We only accept payments with CPR 9052180')

    if format != '521':
        raise InvalidFormatValue('We only accept payments with Format 521')

    payment_identification = _calculate_payment_identification_notebook_60(
        payment_limit_date, suffix
    )

    reference_number_with_control_digits = _calculate_reference_number_with_control_digits_notebook_60( # noqa
        sender,
        reference_number,
        payment_identification,
        quantity,
    )

    payment_code = _build_payment_code_notebook_60(
        sender, reference_number, payment_identification, quantity)


    # Message overrides

    message_1 = ''
    if 'message_1' in extra:
        message_1 = MESSAGE_1_TEMPLATE.format(
            es=extra.get('message_1').get('es', ''),
            eu=extra.get('message_1').get('eu', ''),
        )

    message_2 = ''
    if 'message_2' in extra:
        message_2 = MESSAGE_2_TEMPLATE.format(
            es=extra.get('message_2').get('es', ''),
            eu=extra.get('message_2').get('eu', ''),
        )

    message_3 = ''
    if 'message_3' in extra:
        message_3 = MESSAGE_3_TEMPLATE.format(
            es=extra.get('message_3').get('es', ''),
            eu=extra.get('message_3').get('eu', ''),
        )

    message_4 = ''
    if 'message_4' in extra:
        message_4 = MESSAGE_4_TEMPLATE.format(
            es=extra.get('message_4').get('es', ''),
            eu=extra.get('message_4').get('eu', ''),
        )

    message_payment_title = ''
    if 'message_payment_title' in extra:
        message_payment_title = MESSAGE_PAYMENT_TITLE.format(
            es=extra.get('message_payment_title').get('es', ''),
            eu=extra.get('message_payment_title').get('eu', ''),
        )

    mipago_payment_description = ''
    if 'mipago_payment_description' in extra:
        mipago_payment_description = MESSAGE_PAYMENT_DESCRIPTION.format(
            es=extra.get('mipago_payment_description').get('es', ''),
            eu=extra.get('mipago_payment_description').get('eu', ''),
        )

    logo_urls = ''
    if 'logo_1_url' in extra:
        logo_urls += LOGO_1_TEMPLATE.format(
            url=extra.get('logo_1_url', '')
        )

    if 'logo_2_url' in extra:
        logo_urls += LOGO_2_TEMPLATE.format(
            url=extra.get('logo_2_url', '')
        )

    if logo_urls:
        logo_urls = LOGO_WRAPPER_TEMPLATE.format(
            data=logo_urls,
        )

    pdf_xslt_url = ''
    if 'pdf_xslt_url' in extra:
        pdf_xslt_url = PDF_XSLT_TEMPLATE.format(
            url=extra.get('pdf_xslt_url', '')
        )

    initialization_xml = INITIALIZATION_XML.format(
        code=payment_code,
        cpr=cpr,
        suffix=suffix,
        quantity=quantity,
        payment_identification=payment_identification,
        end_date=payment_limit_date.strftime('%d%m%y'),
        format=format,
        sender=sender,
        reference_with_control=reference_number_with_control_digits,
        reference=reference_number,
        message_1=message_1,
        message_2=message_2,
        message_3=message_3,
        message_4=message_4,
        message_payment_title=message_payment_title,
        mipago_payment_description=mipago_payment_description,
        citizen_name=extra.get('citizen_name', ''),
        citizen_surname_1=extra.get('citizen_surname_1', ''),
        citizen_surname_2=extra.get('citizen_surname_2', ''),
        citizen_city=extra.get('citizen_city', ''),
        citizen_nif=extra.get('citizen_nif', ''),
        citizen_address=extra.get('citizen_address', ''),
        citizen_postal_code=extra.get('citizen_postal_code', ''),
        citizen_territory=extra.get('citizen_territory', ''),
        citizen_country=extra.get('citizen_country', ''),
        citizen_phone=extra.get('citizen_phone', ''),
        citizen_email=extra.get('citizen_email', ''),
        logo_urls=logo_urls,
        pdf_xslt_url=pdf_xslt_url,
    )

    with open('/tmp/send.xml', 'w') as fp:
        fp.write(initialization_xml)

    response = requests.post(
        INITIALIZATION_URL,
        data={'xmlRPC': initialization_xml}
    )

    registered_payment_id, error = _parse_initialization_response(response.content) # noqa

    if registered_payment_id is not None:
        payment_mode_string = ''
        for payment_mode in payment_modes:
            payment_mode_string += "<paymentMode oid='{}'/>".format(payment_mode) # noqa

        presentation_request_data = PRESENTATION_XML.format(
            language=language,
            payment_mode=payment_mode_string
        )

        protocol_data = PROTOCOL_DATA_XML.format(
            return_url=return_url
        )
        response = requests.post(
            SERVICE_URL,
            data={
                'p12iOidsPago': registered_payment_id,
                'p12iPresentationRequestData': presentation_request_data,
                'p12iProtocolData': protocol_data
            }
        )

        return response.text, registered_payment_id

    raise InvalidRegistration(error)



# -*- coding: utf-8 -*-
""" includes some constants used by the library"""


TEST_ENVIRON_INITIALIZATION_URL = 'https://www.testpago.euskadi.eus/p12gWar/p12gRPCDispatcherServlet' # noqa

TEST_ENVIRON_SERVICE_URL = 'https://www.testpago.euskadi.eus/p12iWar/p12iRPCDispatcherServlet' # noqa

PROD_ENVIRON_INITIALIZATION_URL = 'https://www.euskadi.eus/p12gWar/p12gRPCDispatcherServlet' # noqa

PROD_ENVIRON_SERVICE_URL = 'https://www.euskadi.eus/p12iWar/p12iRPCDispatcherServlet' # noqa

INITIALIZATION_XML = '''<?xml version="1.0" encoding="ISO-8859-1" ?>
<rpcCall module="rpcCoreDaemon">
    <function name="doExecInitializePayment">
        <param name="param0" type="XML">
            <paymentRequestData>
                <peticionesPago>
                    <peticionPago id='0'>

                        <liquidacion>

                            <mensajes>
                                {message_1}

                                {message_2}

                                {message_3}

                                {message_4}

                            </mensajes>

                            {logo_urls}

                            {pdf_xslt_url}

                            <paymentGatewayVersion>3</paymentGatewayVersion>
                        </liquidacion>

                        {message_payment_title}

                        <expediente>
                            <codigo>{reference}</codigo>
                            <tercero>
                                <primerApellido>{citizen_surname_1}</primerApellido>
                                <segundoApellido>{citizen_surname_2}</segundoApellido>
                                <razonSocial>{citizen_name}</razonSocial>
                                <dniNif>{citizen_nif}</dniNif>
                                <calle>{citizen_address}</calle>
                                <municipio>{citizen_city}</municipio>
                                <codigoPostal>{citizen_postal_code}</codigoPostal>
                                <territorio>{citizen_territory}</territorio>
                                <pais>{citizen_country}</pais>
                                <datosAdicionales>
                                    <datoAdicional id="telefono">
                                        <valor>{citizen_phone}</valor>
                                    </datoAdicional>
                                    <datoAdicional id="email">
                                        <valor>{citizen_email}</valor>
                                    </datoAdicional>

                                </datosAdicionales>
                            </tercero>
                        </expediente>

                        <conceptos>
                            <conceptoPeticion>
                                <numeroLinea>1</numeroLinea>
                                <baseImponible>0</baseImponible>
                                {mipago_payment_description}
                                <unidades>1</unidades>
                                <tieneIVARepercutido>false</tieneIVARepercutido>
                                <IVARepercutido>false</IVARepercutido>
                                <tipoIVA>0</tipoIVA>
                                <importe>{quantity}</importe>
                                <importeIVA>0</importeIVA>
                            </conceptoPeticion>
                        </conceptos>

                        <datosPago>

                            <codigo>{code}</codigo>
                            <cpr>{cpr}</cpr>
                            <tipo>{suffix}</tipo>
                            <periodosPago>
                                <periodoPago id='periodoNormal'>
                                    <importe>{quantity}</importe>
                                    <validarFechaFin>true</validarFechaFin>
                                    <identificacion>{payment_identification}</identificacion>
                                    <fechaFin>{end_date}</fechaFin>
                                    <activo>false</activo>
                                </periodoPago>
                            </periodosPago>
                            <formato>{format}</formato>
                            <emisor>{sender}</emisor>
                            <validar>1</validar>
                            <referencia>{reference_with_control}</referencia>

                        </datosPago>
                    </peticionPago>
                </peticionesPago>
            </paymentRequestData>
        </param>
    </function>
</rpcCall>
'''

PRESENTATION_XML = '''<presentationRequestData>
    <idioma>{language}</idioma>
    <paymentModes>
        {payment_mode}
    </paymentModes>
</presentationRequestData>
'''

PROTOCOL_DATA_XML = '''<protocolData>
    <urls>
        <url id='urlVuelta'><![CDATA[{return_url}]]></url>
    </urls>
</protocolData>
'''

MESSAGE_PAYMENT_TITLE = '''<descripcion>
    <eu>{eu}</eu>
    <es>{es}</es>
</descripcion>'''

MESSAGE_PAYMENT_DESCRIPTION = '''<descripcion>
    <eu>{eu}</eu>
    <es>{es}</es>
</descripcion>'''


MESSAGE_1_TEMPLATE = """<mensaje id="1">
    <texto>
        <eu>{eu}</eu>
        <es>{es}</es>
    </texto>
</mensaje>"""

MESSAGE_2_TEMPLATE = """<mensaje id="2">
    <texto>
        <eu>{eu}</eu>
        <es>{es}</es>
    </texto>
</mensaje>"""

MESSAGE_3_TEMPLATE = """<mensaje id="3">
    <texto>
        <eu>{eu}</eu>
        <es>{es}</es>
    </texto>
</mensaje>"""

MESSAGE_4_TEMPLATE = """<mensaje id="4">
    <texto>
        <eu>{eu}</eu>
        <es>{es}</es>
    </texto>
</mensaje>"""

LOGO_1_TEMPLATE = """<imagen id="logo1">
        <url><![CDATA[{url}]]></url>
</imagen>"""


LOGO_2_TEMPLATE = """<imagen id="logo2">
        <url><![CDATA[{url}]]></url>
</imagen>"""

LOGO_WRAPPER_TEMPLATE = """<imagenes>
    {data}
</imagenes>"""

PDF_XSLT_TEMPLATE = """<urlPlantilla>
    <![CDATA[{url}]]>
</urlPlantilla>"""



# -*- coding: utf-8 -*-


class InvalidCPRValue(Exception):
    """Raised when the used CPR value is not valid. """
    pass


class InvalidReferenceNumber(Exception):
    """Raised when the format of the reference_number is not valid. """
    pass


class InvalidFormatValue(Exception):
    """Raised when the used Format value is not valid. """
    pass


class InvalidRegistration(Exception):
    """Raised when the registration of the payment on the Government platform
       is invalid and has created an error
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['requests']

setup_requirements = []

test_requirements = []

setup(
    author="Mikel Larreategi",
    author_email='mlarreategi@codesyntax.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    description="Python package to make payment requests with Basque Government payment service",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='pymipago',
    name='pymipago',
    packages=find_packages(include=['pymipago']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/codesyntax/pymipago',
    version='1.0b5',
    zip_safe=False,
)
